@@ -1,5 +1,7 @@
+/** @file chm_common.c
+ */
 #include "chm_common.h"
-#include "Mutils.h"
+// -> Mutils.h
 
 Rboolean isValid_Csparse(SEXP x); /* -> Csparse.c */
 
@@ -10,8 +12,8 @@ SEXP get_SuiteSparse_version() {
     return ans;
 }
 
-cholmod_common c;
-cholmod_common cl;
+cholmod_common c; // for cholmod_<method>  (..)
+cholmod_common cl;// for cholmod_l_<method>(..)
 
 SEXP chm_common_env;
 static SEXP dboundSym, grow0Sym, grow1Sym, grow2Sym, maxrankSym,
@@ -96,12 +98,31 @@ SEXP CHM_set_common_env(SEXP rho) {
     return R_NilValue;
 }
 
+/** @brief stype := "symmetry type".
+ *
+ *  ./CHOLMOD/Include/cholmod_core.h says about  'int stype' entry of cholmod_sparse_struct:
+ *    ------------------------------
+ * 0:  matrix is "unsymmetric": use both upper and lower triangular parts
+ *     (the matrix may actually be symmetric in pattern and value, but
+ *     both parts are explicitly stored and used).  May be square or
+ *     rectangular.
+ * >0: matrix is square and symmetric, use upper triangular part.
+ *     Entries in the lower triangular part are ignored.
+ * <0: matrix is square and symmetric, use lower triangular part.
+ *     Entries in the upper triangular part are ignored.
+ */
 static int stype(int ctype, SEXP x)
 {
     if ((ctype % 3) == 1) return (*uplo_P(x) == 'U') ? 1 : -1;
     return 0;
 }
 
+/** @brief xtype: the _kind_ of numeric (think "x slot") of Cholmod sparse matrices.
+  #define CHOLMOD_PATTERN 0	 pattern only, no numerical values
+  #define CHOLMOD_REAL    1	 a real matrix
+  #define CHOLMOD_COMPLEX 2	 a complex matrix (ANSI C99 compatible)
+  #define CHOLMOD_ZOMPLEX 3	 a complex matrix (MATLAB compatible)
+*/
 static int xtype(int ctype)
 {
     switch(ctype / 3) {
@@ -172,12 +193,12 @@ static void chm2Ralloc(CHM_SP dest, CHM_SP src)
     /* R_alloc the vector storage for dest and copy the contents from src */
     np1 = src->ncol + 1;
     nnz = (int) cholmod_nnz(src, &c);
-    dest->p = (void*) Memcpy((int*)R_alloc(sizeof(int), np1),
+    dest->p = (void*) Memcpy((int*)R_alloc(np1, sizeof(int)),
 			     (int*)(src->p), np1);
-    dest->i = (void*) Memcpy((int*)R_alloc(sizeof(int), nnz),
+    dest->i = (void*) Memcpy((int*)R_alloc(nnz, sizeof(int)),
 			     (int*)(src->i), nnz);
     if(src->xtype)
-	dest->x = (void*) Memcpy((double*)R_alloc(sizeof(double), nnz),
+	dest->x = (void*) Memcpy((double*)R_alloc(nnz, sizeof(double)),
 				 (double*)(src->x), nnz);
 }
 
@@ -193,12 +214,12 @@ static void chTr2Ralloc(CHM_TR dest, CHM_TR src)
 
     /* R_alloc the vector storage for dest and copy the contents from src */
     nnz = src->nnz;
-    dest->i = (void*) Memcpy((int*)R_alloc(sizeof(int), nnz),
+    dest->i = (void*) Memcpy((int*)R_alloc(nnz, sizeof(int)),
 			     (int*)(src->i), nnz);
-    dest->j = (void*) Memcpy((int*)R_alloc(sizeof(int), nnz),
+    dest->j = (void*) Memcpy((int*)R_alloc(nnz, sizeof(int)),
 			     (int*)(src->j), nnz);
     if(src->xtype)
-	dest->x = (void*) Memcpy((double*)R_alloc(sizeof(double), nnz),
+	dest->x = (void*) Memcpy((double*)R_alloc(nnz, sizeof(double)),
 				 (double*)(src->x), nnz);
 }
 
@@ -226,7 +247,7 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
 {
     static const char *valid[] = { MATRIX_VALID_Csparse, ""};
     int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
-	ctype = Matrix_check_class_etc(x, valid);
+	ctype = R_check_class_etc(x, valid);
     SEXP islot = GET_SLOT(x, Matrix_iSym);
 
     if (ctype < 0) error(_("invalid class of object to as_cholmod_sparse"));
@@ -282,7 +303,7 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
 	CHM_SP eye = cholmod_speye(ans->nrow, ans->ncol, ans->xtype, &c);
 	CHM_SP tmp = cholmod_add(ans, eye, one, one, TRUE, TRUE, &c);
 
-#ifdef DEBUG_Matrix_verbose /* quite often, e.g. in ../tests/indexing.R */
+#ifdef DEBUG_Matrix_verbose /* happens quite often, e.g. in ../tests/indexing.R : */
 	Rprintf("Note: as_cholmod_sparse(<ctype=%d>) - diagU2N\n", ctype);
 #endif
 	chm2Ralloc(ans, tmp);
@@ -304,7 +325,7 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
  * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 Free a
  * @param uploT 0 - not triangular; > 0 upper triangular; < 0 lower
  * @param Rkind - vector type to store for a->xtype == CHOLMOD_REAL,
- *                0 - REAL; 1 - LOGICAL
+ *                0 - REAL; 1 - LOGICAL  [unused for other a->xtype]
  * @param diag character string suitable for the diag slot of a
  *          triangular matrix (not accessed if uploT == 0).
  * @param dn either R_NilValue or an SEXP suitable for the Dimnames slot.
@@ -316,9 +337,15 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
 {
     SEXP ans;
     char *cls = "";/* -Wall */
-    int *dims, nnz, *ansp, *ansi, *aii = (int*)(a->i), *api = (int*)(a->p),
-	longi = (a->itype) == CHOLMOD_LONG;
-    UF_long *ail = (UF_long*)(a->i), *apl = (UF_long*)(a->p);
+    Rboolean longi = (a->itype) == CHOLMOD_LONG;
+    int *dims, nnz, *ansp, *ansi;
+    // if (longi) :
+    SuiteSparse_long
+	*ail = (SuiteSparse_long*)(a->i),
+	*apl = (SuiteSparse_long*)(a->p);
+    // else  ((a->itype) == CHOLMOD_INT) :
+    int *aii = (int*)(a->i),
+	*api = (int*)(a->p);
 
     PROTECT(dn);  /* dn is usually UNPROTECTed before the call */
 
@@ -332,7 +359,8 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
 	longi ? cholmod_l_free_sparse(&a, &cl) : cholmod_free_sparse(&a, &c); \
     else if (dofree < 0) Free(a)
 
-    switch(a->xtype){
+
+    switch(a->xtype) {
     case CHOLMOD_PATTERN:
 	cls = uploT ? "ntCMatrix": ((a->stype) ? "nsCMatrix" : "ngCMatrix");
 	break;
@@ -356,7 +384,7 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
 	DOFREE_MAYBE;
 	error(_("unknown xtype in cholmod_sparse object"));
     }
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(cls)));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(cls));
 				/* allocate and copy common slots */
     nnz = longi ? cholmod_l_nnz(a, &cl) : cholmod_nnz(a, &c);
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
@@ -364,7 +392,7 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
     ansp = INTEGER(ALLOC_SLOT(ans, Matrix_pSym, INTSXP, a->ncol + 1));
     ansi = INTEGER(ALLOC_SLOT(ans, Matrix_iSym, INTSXP, nnz));
     for (int j = 0; j <= a->ncol; j++) ansp[j] = longi ? (int)(apl[j]) : api[j];
-    for (int p = 0; p < nnz; p++) ansi[p] = longi ? (int)(ail[p]) : aii[p];
+    for (int p = 0; p < nnz; p++)      ansi[p] = longi ? (int)(ail[p]) : aii[p];
 				/* copy data slot if present */
     if (a->xtype == CHOLMOD_REAL) {
 	int i, *m_x;
@@ -404,6 +432,42 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
 }
 #undef DOFREE_MAYBE
 
+
+/**
+* Change the "type" of a cholmod_sparse matrix, i.e. modify it "in place"
+*
+* @param to_xtype requested xtype (pattern, real, complex, zomplex)
+* @param A sparse matrix to change
+* @param Common cholmod's common
+*
+* @return TRUE/FALSE , TRUE iff success
+*/
+Rboolean chm_MOD_xtype(int to_xtype, cholmod_sparse *A, CHM_CM Common) {
+//     *MOD*: shouting, as A is modified in place
+
+/* --------------------------------------------------------------------------
+ * cholmod_sparse_xtype: change the xtype of a sparse matrix
+ * --------------------------------------------------------------------------
+  int cholmod_sparse_xtype
+  (
+      // ---- input ----
+      int to_xtype,	//
+      // ---- in/out ---
+      cholmod_sparse *A, //
+      // ---------------
+      cholmod_common *Common
+  ) ;
+
+  int cholmod_l_sparse_xtype (int, cholmod_sparse *, cholmod_common *) ;
+*/
+    if((A->itype) == CHOLMOD_LONG) {
+	return (Rboolean) cholmod_l_sparse_xtype (to_xtype, A, Common);
+    } else {
+	return (Rboolean) cholmod_sparse_xtype   (to_xtype, A, Common);
+    }
+}
+
+
 /**
  * Populate ans with the pointers from x and modify its scalar
  * elements accordingly. Note that later changes to the contents of
@@ -423,8 +487,8 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
 CHM_TR as_cholmod_triplet(CHM_TR ans, SEXP x, Rboolean check_Udiag)
 {
     static const char *valid[] = { MATRIX_VALID_Tsparse, ""};
-    int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
-	ctype = Matrix_check_class_etc(x, valid);
+    int ctype = R_check_class_etc(x, valid),
+	*dims = INTEGER(GET_SLOT(x, Matrix_DimSym));
     SEXP islot = GET_SLOT(x, Matrix_iSym);
     int m = LENGTH(islot);
     Rboolean do_Udiag = (check_Udiag && ctype % 3 == 2 && (*diag_P(x) == 'U'));
@@ -448,10 +512,10 @@ CHM_TR as_cholmod_triplet(CHM_TR ans, SEXP x, Rboolean check_Udiag)
 	/* diagU2N(.) "in place", similarly to Tsparse_diagU2N [./Tsparse.c]
 	   (but without new SEXP): */
 	int k = m + dims[0];
-	CHM_TR tmp = cholmod_l_copy_triplet(ans, &c);
+	CHM_TR tmp = cholmod_l_copy_triplet(ans, &cl);
 	int *a_i, *a_j;
 
-	if(!cholmod_reallocate_triplet((size_t) k, tmp, &c))
+	if(!cholmod_reallocate_triplet((size_t) k, tmp, &cl))
 	    error(_("as_cholmod_triplet(): could not reallocate for internal diagU2N()"
 		      ));
 
@@ -552,7 +616,7 @@ SEXP chm_triplet_to_SEXP(CHM_TR a, int dofree, int uploT, int Rkind,
 	DOFREE_MAYBE;
 	error(_("unknown xtype in cholmod_triplet object"));
     }
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(cl)));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(cl));
 				/* allocate and copy common slots */
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     dims[0] = a->nrow; dims[1] = a->ncol;
@@ -615,8 +679,8 @@ SEXP chm_triplet_to_SEXP(CHM_TR a, int dofree, int uploT, int Rkind,
 CHM_DN as_cholmod_dense(CHM_DN ans, SEXP x)
 {
 #define _AS_cholmod_dense_1						\
-    static const char *valid[] = { MATRIX_VALID_dense, ""};		\
-    int dims[2], ctype = Matrix_check_class_etc(x, valid), nprot = 0;	\
+    static const char *valid[] = { MATRIX_VALID_ge_dense, ""};		\
+    int dims[2], ctype = R_check_class_etc(x, valid), nprot = 0;	\
 									\
     if (ctype < 0) {		/* not a classed matrix */		\
 	if (isMatrix(x)) Memcpy(dims, INTEGER(getAttrib(x, R_DimSymbol)), 2); \
@@ -637,7 +701,7 @@ CHM_DN as_cholmod_dense(CHM_DN ans, SEXP x)
 				/* dimensions and nzmax */		\
     ans->d = ans->nrow = dims[0];					\
     ans->ncol = dims[1];						\
-    ans->nzmax = dims[0] * dims[1];					\
+    ans->nzmax = ((size_t)dims[0]) * dims[1];				\
 				/* set the xtype and any elements */	\
     switch(ctype / 2) {							\
     case 0: /* "d" */							\
@@ -671,7 +735,7 @@ CHM_DN as_cholmod_dense(CHM_DN ans, SEXP x)
 /* version of as_cholmod_dense() that produces a cholmod_dense matrix
  * with REAL 'x' slot -- i.e. treats "nMatrix" as "lMatrix" -- as only difference;
  * Not just via a flag in as_cholmod_dense() since that has fixed API */
-CHM_DN as_cholmod_x_dense(cholmod_dense *ans, SEXP x)
+CHM_DN as_cholmod_x_dense(CHM_DN ans, SEXP x)
 {
     _AS_cholmod_dense_1;
 
@@ -687,6 +751,28 @@ CHM_DN as_cholmod_x_dense(cholmod_dense *ans, SEXP x)
 #undef _AS_cholmod_dense_1
 #undef _AS_cholmod_dense_2
 
+/**
+* Transpose a cholmod_dense matrix  ("too trivial to be in CHOLMOD?")
+*
+* @param ans (pointer to) already allocated result of correct dimension
+* @param x   (pointer to) cholmod_dense matrix to be transposed
+*
+*/
+void chm_transpose_dense(CHM_DN ans, CHM_DN x)
+{
+    if (x->xtype != CHOLMOD_REAL)
+	error(_("chm_transpose_dense(ans, x) not yet implemented for %s different from %s"),
+	      "x->xtype", "CHOLMOD_REAL");
+    double *xx = x->x, *ansx = ans->x;
+    // Inspired from R's do_transpose() in .../R/src/main/array.c :
+    int i,j, nrow = x->nrow, len = x->nzmax, l_1 = len-1;
+    for (i = 0, j = 0; i < len; i++, j += nrow) {
+	if (j > l_1) j -= l_1;
+	ansx[i] = xx[j];
+    }
+    return;
+}
+
 void R_cholmod_error(int status, const char *file, int line, const char *message)
 {
     CHM_restore_common(); /* restore any setting that may have been changed */
@@ -748,20 +834,19 @@ int R_cholmod_start(CHM_CM c)
  * @param a matrix to be converted
  * @param dofree 0 - don't free a; > 0 cholmod_free a; < 0 Free a
  * @param Rkind type of R matrix to be generated (special to this function)
- * @param dn   -- dimnames [list(.,.) or NULL]
+ * @param dn   -- dimnames [list(.,.) or NULL;  __already__ transposed when transp is TRUE ]
+ * @param transp Rboolean, if TRUE, the result must be a copy of  t(a), i.e., "a transposed"
  *
  * @return SEXP containing a copy of a
  */
-
-/* FIXME: should also have args  (int uploST, char *diag) */
-
-SEXP chm_dense_to_SEXP(CHM_DN a, int dofree, int Rkind, SEXP dn)
+SEXP chm_dense_to_SEXP(CHM_DN a, int dofree, int Rkind, SEXP dn, Rboolean transp)
 {
+/* FIXME: should also have args  (int uploST, char *diag) */
     SEXP ans;
     char *cl = ""; /* -Wall */
     int *dims, ntot;
 
-    PROTECT(dn); /* << (why? -- just cut&paste from chm_dense_to_mat.. below*/
+    PROTECT(dn); // <-- no longer protected in caller
 
 #define DOFREE_de_MAYBE				\
     if (dofree > 0) cholmod_free_dense(&a, &c);	\
@@ -789,25 +874,47 @@ SEXP chm_dense_to_SEXP(CHM_DN a, int dofree, int Rkind, SEXP dn)
 	error(_("unknown xtype"));
     }
 
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(cl)));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(cl));
 				/* allocate and copy common slots */
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
-    dims[0] = a->nrow; dims[1] = a->ncol;
-    ntot = dims[0] * dims[1];
+    if(transp) {
+	dims[1] = a->nrow; dims[0] = a->ncol;
+    } else {
+	dims[0] = a->nrow; dims[1] = a->ncol;
+    }
+    ntot = ((size_t)dims[0]) * dims[1];
     if (a->d == a->nrow) { /* copy data slot -- always present in dense(!) */
 	if (a->xtype == CHOLMOD_REAL) {
 	    int i, *m_x;
-	    double *a_x = (double *) a->x;
+	    double *ansx, *a_x = (double *) a->x;
 	    switch(Rkind) {
 	    case 0:
-		Memcpy(REAL(ALLOC_SLOT(ans, Matrix_xSym, REALSXP, ntot)),
-		       a_x, ntot);
+		ansx = REAL(ALLOC_SLOT(ans, Matrix_xSym, REALSXP, ntot));
+		if(transp) {
+		    // Inspired from R's do_transpose() in .../R/src/main/array.c :
+		    int i,j, nrow = a->nrow, len = ntot, l_1 = len-1;
+		    for (i = 0, j = 0; i < len; i++, j += nrow) {
+			if (j > l_1) j -= l_1;
+			ansx[i] = a_x[j];
+		    }
+		} else {
+		    Memcpy(ansx, a_x, ntot);
+		}
 		break;
 	    case -1: /* nge*/
 	    case 1:  /* lge*/
 		m_x = LOGICAL(ALLOC_SLOT(ans, Matrix_xSym, LGLSXP, ntot));
-		for (i=0; i < ntot; i++)
-		    m_x[i] = ISNAN(a_x[i]) ? NA_LOGICAL : (a_x[i] != 0);
+		if(transp) {
+		    // Inspired from R's do_transpose() in .../R/src/main/array.c :
+		    int i,j, nrow = a->nrow, len = ntot, l_1 = len-1;
+		    for (i = 0, j = 0; i < len; i++, j += nrow) {
+			if (j > l_1) j -= l_1;
+			m_x[i] = a_x[j];
+		    }
+		} else {
+		    for (i=0; i < ntot; i++)
+			m_x[i] = ISNAN(a_x[i]) ? NA_LOGICAL : (a_x[i] != 0);
+		}
 		break;
 	    }
 	}
@@ -920,19 +1027,17 @@ CHM_DN numeric_as_chm_dense(CHM_DN ans, double *v, int nr, int nc)
  * elements accordingly. Note that later changes to the contents of
  * ans will change the contents of the SEXP.
  *
- * In most cases this function is called through the macro AS_CHM_FR.
- * It is unusual to call it directly.
- *
  * @param ans an CHM_FR object
  * @param x pointer to an object that inherits from CHMfactor
+ * @param do_check logical indicating if check for correctness should happen
  *
  * @return ans containing pointers to the slots of x.
  */
-CHM_FR as_cholmod_factor(CHM_FR ans, SEXP x)
+CHM_FR as_cholmod_factor3(CHM_FR ans, SEXP x, Rboolean do_check)
 {
     static const char *valid[] = { MATRIX_VALID_CHMfactor, ""};
     int *type = INTEGER(GET_SLOT(x, install("type"))),
-	ctype = Matrix_check_class_etc(x, valid);
+	ctype = R_check_class_etc(x, valid);
     SEXP tmp;
 
     if (ctype < 0) error(_("invalid class of object to as_cholmod_factor"));
@@ -988,11 +1093,29 @@ CHM_FR as_cholmod_factor(CHM_FR ans, SEXP x)
 	ans->next = INTEGER(GET_SLOT(x, install("nxt")));
 	ans->prev = INTEGER(GET_SLOT(x, install("prv")));
     }
-    if (!cholmod_check_factor(ans, &c))
+    if (do_check && !cholmod_check_factor(ans, &c))
 	error(_("failure in as_cholmod_factor"));
     return ans;
 }
 
+// This has been in the Matrix API  ( ../inst/include/Matrix.h
+/**
+ * Populate ans with the pointers from x and modify its scalar
+ * elements accordingly. Note that later changes to the contents of
+ * ans will change the contents of the SEXP.
+ *
+ * In most cases this function is called through the macro AS_CHM_FR.
+ * It is unusual to call it directly.
+ *
+ * @param ans an CHM_FR object
+ * @param x pointer to an object that inherits from CHMfactor
+ *
+ * @return ans containing pointers to the slots of x.
+ */
+CHM_FR as_cholmod_factor(CHM_FR ans, SEXP x) {
+    return as_cholmod_factor3(ans, x, /* do_check = */ TRUE);
+}
+
 
 /**
  * Copy the contents of f to an appropriate CHMfactor object and,
@@ -1033,7 +1156,7 @@ SEXP chm_factor_to_SEXP(CHM_FR f, int dofree)
 	error(_("f->xtype of %d not recognized"), f->xtype);
     }
 
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(class)));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(class));
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     dims[0] = dims[1] = f->n;
 				/* copy component of known length */
