@@ -19,8 +19,8 @@ SEXP xCMatrix_validate(SEXP x)
 {
     /* Almost everything now in Csparse_validate ( ./Csparse.c )
      * *but* the checking of the 'x' slot : */
-    if (length(GET_SLOT(x, Matrix_iSym)) !=
-	length(GET_SLOT(x, Matrix_xSym)))
+    if (xlength(GET_SLOT(x, Matrix_iSym)) !=
+	xlength(GET_SLOT(x, Matrix_xSym)))
 	return mkString(_("lengths of slots 'i' and 'x' must match"));
 
     return ScalarLogical(1);
@@ -31,8 +31,8 @@ SEXP xRMatrix_validate(SEXP x)
 {
     /* Almost everything now in Rsparse_validate ( ./Csparse.c )
      * *but* the checking of the 'x' slot : */
-    if (length(GET_SLOT(x, Matrix_jSym)) !=
-	length(GET_SLOT(x, Matrix_xSym)))
+    if (xlength(GET_SLOT(x, Matrix_jSym)) !=
+	xlength(GET_SLOT(x, Matrix_xSym)))
 	return mkString(_("lengths of slots 'j' and 'x' must match"));
 
     return ScalarLogical(1);
@@ -46,9 +46,9 @@ SEXP compressed_to_TMatrix(SEXP x, SEXP colP)
     /* however, for Csparse, we now effectively use the cholmod-based
      * Csparse_to_Tsparse() in ./Csparse.c ; maybe should simply write
      * an  as_cholmod_Rsparse() function and then do "as there" ...*/
-    SEXP indSym = col ? Matrix_iSym : Matrix_jSym,
-	ans, indP = GET_SLOT(x, indSym),
-	pP = GET_SLOT(x, Matrix_pSym);
+    SEXP indSym = col ? Matrix_iSym : Matrix_jSym, ans,
+	indP = PROTECT(GET_SLOT(x, indSym)),
+	  pP = PROTECT(GET_SLOT(x, Matrix_pSym));
     int npt = length(pP) - 1;
     char *ncl = strdup(class_P(x));
     static const char *valid[] = { MATRIX_VALID_Csparse, MATRIX_VALID_Rsparse, ""};
@@ -59,7 +59,7 @@ SEXP compressed_to_TMatrix(SEXP x, SEXP colP)
 
     /* replace 'C' or 'R' with 'T' :*/
     ncl[2] = 'T';
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(ncl)));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(ncl));
 
     slot_dup(ans, x, Matrix_DimSym);
     if((ctype / 3) % 4 != 2) /* not n..Matrix */
@@ -76,7 +76,7 @@ SEXP compressed_to_TMatrix(SEXP x, SEXP colP)
 		  INTEGER(ALLOC_SLOT(ans, col ? Matrix_jSym : Matrix_iSym,
 				     INTSXP, length(indP))));
     free(ncl);
-    UNPROTECT(1);
+    UNPROTECT(3);
     return ans;
 }
 
@@ -94,7 +94,7 @@ SEXP R_to_CMatrix(SEXP x)
 
     /* replace 'R' with 'C' : */
     ncl[2] = 'C';
-    PROTECT_WITH_INDEX(ans = NEW_OBJECT(MAKE_CLASS(ncl)), &ipx);
+    PROTECT_WITH_INDEX(ans = NEW_OBJECT_OF_CLASS(ncl), &ipx);
 
     a_dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     /* reversed dim() since we will transpose: */
@@ -130,8 +130,8 @@ SEXP compressed_non_0_ij(SEXP x, SEXP colP)
 {
     int col = asLogical(colP); /* 1 if "C"olumn compressed;  0 if "R"ow */
     SEXP ans, indSym = col ? Matrix_iSym : Matrix_jSym;
-    SEXP indP = GET_SLOT(x, indSym),
-	pP = GET_SLOT(x, Matrix_pSym);
+    SEXP indP = PROTECT(GET_SLOT(x, indSym)),
+	 pP   = PROTECT(GET_SLOT(x, Matrix_pSym));
     int i, *ij;
     int nouter = INTEGER(GET_SLOT(x, Matrix_DimSym))[col ? 1 : 0],
 	n_el   = INTEGER(pP)[nouter]; /* is only == length(indP), if the
@@ -148,7 +148,7 @@ SEXP compressed_non_0_ij(SEXP x, SEXP colP)
 	for(i = 0; i < n_el; i++)
 	    ij[i + n_el] = INTEGER(indP)[i];
 
-    UNPROTECT(1);
+    UNPROTECT(3);
     return ans;
 }
 
@@ -227,7 +227,7 @@ SEXP dgCMatrix_QR(SEXP Ap, SEXP order, SEXP keep_dimnames)
     R_CheckStack();
 
     if (m < n) error(_("A must have #{rows} >= #{columns}")) ;
-    SEXP ans = PROTECT(NEW_OBJECT(MAKE_CLASS("sparseQR")));
+    SEXP ans = PROTECT(NEW_OBJECT_OF_CLASS("sparseQR"));
     int *dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     dims[0] = m; dims[1] = n;
     css *S = cs_sqr(ord, A, 1);	/* symbolic QR ordering & analysis*/
@@ -317,7 +317,7 @@ SEXP dgCMatrix_QR(SEXP Ap, SEXP order, SEXP keep_dimnames)
 SEXP dgCMatrix_SPQR(SEXP Ap, SEXP ordering, SEXP econ, SEXP tol)
 {
 /* SEXP ans = PROTECT(allocVector(VECSXP, 4)); */
-    SEXP ans = PROTECT(NEW_OBJECT(MAKE_CLASS("SPQR")));
+    SEXP ans = PROTECT(NEW_OBJECT_OF_CLASS("SPQR"));
 
     CHM_SP A = AS_CHM_SP(Ap), Q, R;
     SuiteSparse_long *E, rank;/* not always = int   FIXME  (Windows_64 ?) */
@@ -398,7 +398,7 @@ void install_lu(SEXP Ap, int order, double tol, Rboolean err_sing, Rboolean keep
     N->U = cs_transpose(D, 1);
     cs_spfree(D);
     p = cs_pinv(N->pinv, n);	/* p=pinv' */
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS("sparseLU")));
+    ans = PROTECT(NEW_OBJECT_OF_CLASS("sparseLU"));
     dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     dims[0] = n; dims[1] = n;
     SEXP dn; Rboolean do_dn = FALSE;
